#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ipp.h>



#define NRTAPS_MAX	17000
#define NRSAM		1
#define ERRMAX_REL	((float)1e-2)
#define ERRMAX_ABS	((float)1e-2)



float EmulateIpp_FirOne( float x, IppsFIRState_32f* pFir, int firLen )
{
	//int firLen = (int)(*((unsigned int*)pFir+0x06));	// Only seems to work on 64b - It's safer to pass plain length through interface.

	if ( (firLen<1) || (firLen>=NRTAPS_MAX) )
	{
		printf( "Error: Could not get Fir length\n" );
		exit(-1);
	}

	float* dline = ippsMalloc_32f( firLen );
	float* taps  = ippsMalloc_32f( firLen );
	//float* dline = new float [firLen];
	//float* taps  = new float [firLen];

	ippsFIRGetDlyLine_32f( pFir, dline );
	ippsFIRGetTaps_32f	 ( pFir, taps );

	float t0 = 0.f;
	t0 += taps[0] * x;
	for (int i=1; i<firLen; i++)
	{
		t0 += taps[i] * dline[i-1];
	}

	ippsFree( dline );
	ippsFree( taps  );
	//delete [] dline;
	//delete [] taps;

	return t0;
}



int TestIpp( int nrTaps )
{
    IppStatus status;
    IppsFIRState_32f *pFir;
	float r0, r1, r2;

	float* xx	= ippsMalloc_32f( NRSAM );
	float* yy	= ippsMalloc_32f( NRSAM );
	float* taps	= ippsMalloc_32f( nrTaps );
	float* dline	= ippsMalloc_32f( nrTaps );

	for ( int i=0; i<NRSAM; i++)
	{
		xx[i] = (float)(rand()%20 -10) *100.f/10.f;
		yy[i] = 0.f;
	}

	for ( int i=0; i<nrTaps; i++)
	{
		taps[i]  = (float)(rand()%20 -10) * 1.f/10.f;;
		dline[i] = (float)(rand()%20 -10) *10.f/10.f;;
	}

	ippsFIRInitAlloc_32f( &pFir, taps, nrTaps, NULL );


	// Test.
	ippsFIRSetTaps_32f		( taps, pFir  );
	ippsFIRSetDlyLine_32f	( pFir, dline );
	r0 = EmulateIpp_FirOne( *xx, pFir, nrTaps );

	ippsFIRSetTaps_32f		( taps, pFir  );
	ippsFIRSetDlyLine_32f	( pFir, dline );
    status = ippsFIR_32f( xx, &r1, 1, pFir );

	ippsFIRSetTaps_32f		( taps, pFir  );
	ippsFIRSetDlyLine_32f	( pFir, dline );
    status = ippsFIROne_32f( *xx, &r2, pFir );

	// Verify.
	// Assume EmulatedFir is correct, use it as reference.
	int retval = 0;
	float r1errRel = abs(r1-r0)/abs(r0);
	float r2errRel = abs(r2-r0)/abs(r0);
	float r1errAbs = r1 - r0;
	float r2errAbs = r2 - r0;
	if ( (abs(r1errAbs)>ERRMAX_ABS) && (r1errRel>ERRMAX_REL) )
	{
		retval |= 1;
		//printf( "%i (%f, %f), ", nrTaps, r0, r1 );
	}
	if ( (abs(r2errAbs)>ERRMAX_ABS) && (r2errRel>ERRMAX_REL) )
	{
		retval |= 2;
		//printf( "%i (%f, %f), ", nrTaps, r0, r2 );
	}

    ippsFIRFree_32f( pFir );
	ippsFree( taps	);
	ippsFree( dline );
	ippsFree( xx	);
	ippsFree( yy	);

	return retval;
}



int main()
{
	int errCode;

	printf( "Testing IppFir and IppFirOne for filter lengths in the rage [1...%i]:\n", NRTAPS_MAX-1 );

	//*
	printf( "IppFirOne():\n" );
	int cntErr = 0;
	for (int i=1; i<NRTAPS_MAX; i++)
	{
		errCode = TestIpp( i );
		if (errCode & 2)
		{
			//printf( "%i, ", i );
			cntErr++;
		}
	}
	printf( "\n%i out of %i filter lengths don't work.\n\n", cntErr, NRTAPS_MAX-1 );
	//*/

	//*
	printf( "IppFir():\n" );
	cntErr = 0;
	for (int i=1; i<NRTAPS_MAX; i++)
	{
		errCode = TestIpp( i );
		if (errCode & 1)
		{
			//printf( "%i, ", i );
			cntErr++;
		}
	}
	printf( "\n%i out of %i filter lengths don't work.\n", cntErr, NRTAPS_MAX-1 );
	//*/

	return 0;
}
